@@ -1,19 +1,8 @@
-/********************************************************************
-Copyright © 2019 Roman Gilg <subdiff@gmail.com>
+/*
+    SPDX-FileCopyrightText: 2019 Roman Gilg <subdiff@gmail.com>
 
-This program is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2 of the License, or
-(at your option) any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*********************************************************************/
+    SPDX-License-Identifier: GPL-2.0-or-later
+*/
 #include "config_handler.h"
 
 #include "kcm_screen_debug.h"
@@ -23,6 +12,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <kscreen/getconfigoperation.h>
 #include <kscreen/output.h>
 
+#include <QGuiApplication>
 #include <QRect>
 
 using namespace KScreen;
@@ -97,6 +87,7 @@ void ConfigHandler::initOutput(const KScreen::OutputPtr &output)
 
 void ConfigHandler::updateInitialData()
 {
+    m_previousConfig = m_initialConfig->clone();
     m_initialRetention = getRetention();
     connect(new GetConfigOperation(), &GetConfigOperation::finished, this, [this](ConfigOperation *op) {
         if (op->hasError()) {
@@ -111,6 +102,11 @@ void ConfigHandler::updateInitialData()
     });
 }
 
+bool ConfigHandler::shouldTestNewSettings()
+{
+    return checkSaveandTestCommon(false);
+}
+
 void ConfigHandler::checkNeedsSave()
 {
     if (m_config->supportedFeatures() & KScreen::Config::Feature::PrimaryDisplay) {
@@ -129,40 +125,45 @@ void ConfigHandler::checkNeedsSave()
         Q_EMIT needsSaveChecked(true);
         return;
     }
+    Q_EMIT needsSaveChecked(checkSaveandTestCommon(true));
+}
 
-    for (const auto &output : m_config->connectedOutputs()) {
+bool ConfigHandler::checkSaveandTestCommon(bool isSaveCheck)
+{
+    const auto outputs = m_config->connectedOutputs();
+    for (const auto &output : outputs) {
         const QString hash = output->hashMd5();
-        for (const auto &initialOutput : m_initialConfig->outputs()) {
-            if (hash != initialOutput->hashMd5()) {
+        const auto configs = m_initialConfig->outputs();
+        for (const auto &config : configs) {
+            if (hash != config->hashMd5()) {
                 continue;
             }
-            bool needsSave = false;
-            if (output->isEnabled() != initialOutput->isEnabled()) {
-                needsSave = true;
+
+            if (output->isEnabled() != config->isEnabled()) {
+                return true;
             }
+
             // clang-format off
             if (output->isEnabled()) {
-                needsSave |= output->currentModeId() !=
-                                initialOutput->currentModeId()
-                             || output->pos() != initialOutput->pos()
-                             || output->scale() != initialOutput->scale()
-                             || output->rotation() != initialOutput->rotation()
-                             || output->replicationSource() != initialOutput->replicationSource()
-                             || autoRotate(output) != m_initialControl->getAutoRotate(output)
-                             || autoRotateOnlyInTabletMode(output)
-                                    != m_initialControl->getAutoRotateOnlyInTabletMode(output)
-                             || output->overscan() != initialOutput->overscan()
-                             || output->vrrPolicy() != initialOutput->vrrPolicy();
+                bool realScaleChange = output->scale() != config->scale();
+                bool scaleChanged = isSaveCheck ?  realScaleChange : qGuiApp->platformName() == QLatin1String("wayland") ? realScaleChange : false;
+                if ( output->currentModeId() != config->currentModeId()
+                    || output->pos() != config->pos()
+                    || scaleChanged
+                    || output->rotation() != config->rotation()
+                    || output->replicationSource() != config->replicationSource()
+                    || autoRotate(output) != m_initialControl->getAutoRotate(output)
+                    || autoRotateOnlyInTabletMode(output) != m_initialControl->getAutoRotateOnlyInTabletMode(output)
+                    || output->overscan() != config->overscan()
+                    || output->vrrPolicy() != config->vrrPolicy()
+                    || output->rgbRange() != config->rgbRange()) {
+                        return true;
+                    }
             }
             // clang-format on
-            if (needsSave) {
-                Q_EMIT needsSaveChecked(true);
-                return;
-            }
-            break;
         }
     }
-    Q_EMIT needsSaveChecked(false);
+    return false;
 }
 
 QSize ConfigHandler::screenSize() const
@@ -343,6 +344,16 @@ void ConfigHandler::setVrrPolicy(const KScreen::OutputPtr &output, KScreen::Outp
     m_control->setVrrPolicy(output, value);
 }
 
+KScreen::Output::RgbRange ConfigHandler::rgbRange(const KScreen::OutputPtr &output) const
+{
+    return m_control->getRgbRange(output);
+}
+
+void ConfigHandler::setRgbRange(const KScreen::OutputPtr &output, KScreen::Output::RgbRange value)
+{
+    m_control->setRgbRange(output, value);
+}
+
 void ConfigHandler::writeControl()
 {
     if (!m_control) {
